@@ -14,14 +14,14 @@ use vars qw( $ERROR );
 use MARC::Field;
 use Carp qw(croak);
 
-=head1 VERSION 1.38
+=head1 VERSION
 
-    $Id: Record.pm,v 1.81 2004/03/16 21:52:58 petdance Exp $
+Version 2.0.1
 
 =cut
 
 use vars qw( $VERSION );
-$VERSION = '1.38';
+$VERSION = '2.0.1';
 
 use Exporter;
 use vars qw( @ISA @EXPORTS @EXPORT_OK );
@@ -38,10 +38,6 @@ use constant LEADER_LEN => 24;
 Module for handling MARC records as objects.  The file-handling stuff is
 in MARC::File::*.
 
-=head1 EXPORT
-
-None.
-
 =head1 ERROR HANDLING
 
 Any errors generated are stored in C<$MARC::Record::ERROR>.
@@ -59,7 +55,6 @@ the MARC::File::* modules to read from a file.
 
 sub new {
     my $class = shift;
-    $class = ref($class) || $class; # Handle cloning
     my $self = {
         _leader => ' ' x 24,
         _fields => [],
@@ -186,8 +181,9 @@ sub fields() {
 
 =head2 field( I<tagspec(s)> )
 
-Returns a list of tags that match the field specifier, or in scalar
-context, just the first matching tag.
+Returns a list of tags that match the field specifier, or an empty
+list if nothing matched.  In scalar context, returns the first
+matching tag, or undef if nothing matched.
 
 The field specifier can be a simple number (i.e. "245"), or use the "."
 notation of wildcarding (i.e. subject tags are "6..").
@@ -219,6 +215,7 @@ sub field {
         } # for $maybe
     } # for $tag
 
+    return unless wantarray;
     return @list;
 }
 
@@ -248,7 +245,7 @@ sub subfield {
 
 sub _all_parms_are_fields {
     for ( @_ ) {
-        return 0 unless ref($_) eq 'MARC::Field';
+        return 0 unless UNIVERSAL::isa($_, 'MARC::Field');
     }
     return 1;
 }
@@ -330,13 +327,17 @@ sub insert_fields_after {
     ## find position of $after
     my $fields = $self->{_fields};
     my $pos = 0;
+    my $found = 0;
     foreach my $f (@$fields) {
-        last if ($f == $after);
+        if ($f == $after) {
+            $found = 1;
+            last;
+        }
         $pos++;
     }
 
     ## insert after $after
-    if ($pos+1 >= @$fields) {
+    unless ($found) {
         $self->_warn("Couldn't find field to insert after");
         return;
     }
@@ -420,31 +421,41 @@ sub insert_grouped_field {
 }
 
 
-=head2 delete_field( $field )
+=head2 delete_fields( $field )
 
-Deletes a field from the record.
+Deletes a given list of MARC::Field objects from the the record.
 
-The field must have been retrieved from the record using the
-C<field()> method.  For example, to delete a 526 tag if it exists:
+    # delete all note fields
+    my @notes = $record->field('5..');
+    $record->delete_fields(@notes);
 
-    my $tag526 = $marc->field( "526" );
-    if ( $tag526 ) {
-        $marc->delete_field( $tag526 );
+delete_fields() will return the number of fields that were deleted.
+
+=cut
+
+sub delete_fields {
+    my $self = shift;
+    _all_parms_are_fields(@_) or croak('Arguments must be MARC::Field object');
+    my @fields = @{$self->{_fields}};
+    my $original_count = @fields;
+
+    foreach my $deleter (@_) {
+        @fields = grep { $_ != $deleter } @fields;
     }
+    $self->{_fields} = \@fields;
 
-C<delete_field()> returns the number of fields that were deleted.
-This shouldn't be 0 unless you didn't get the tag properly.
+    return $original_count - @fields;
+}
+
+=head2 delete_field()
+
+Same thing as delete_fields() but only expects a single MARC::Field to be passed
+in. Mainly here for backwards compatibility.
 
 =cut
 
 sub delete_field {
-    my $self = shift;
-    my $deleter = shift;
-    my $list = $self->{_fields};
-
-    my $old_count = @$list;
-    @$list = grep { $_ != $deleter } @$list;
-    return $old_count - @$list;
+    return delete_fields(@_);
 }
 
 =head2 as_usmarc()
@@ -500,6 +511,46 @@ sub leader {
     return $self->{_leader};
 } # leader()
 
+=head2 encoding()
+
+A method for getting/setting the encoding for a record. The encoding for a
+record is determined by position 09 in the leader, which is blank for MARC-8
+encoding, and 'a' for UCS/Unicode. encoding() will return a string, either 
+'MARC-8' or 'UTF-8' appropriately. 
+
+If you want to set the encoding for a MARC::Record object you can use the
+string values:
+
+    $record->encoding( 'UTF-8' );
+
+NOTE: MARC::Record objects created from scratch have an a default encoding
+of MARC-8, which has been the standard for years...but many online catlogs
+and record vendors are migrating to UTF-8.
+
+WARNING: you should be sure your record really does contain valid UTF-8 data
+when you manually set the encoding. 
+
+=cut
+
+sub encoding {
+    my ($self,$arg) = @_;
+    # we basically report from and modify the leader directly
+    my $leader = $self->leader();
+
+    # when setting
+    if ( defined($arg) ) {
+        if ( $arg =~ /UTF-?8/i ) { 
+            substr($leader,9,1) = 'a';
+        }
+        elsif ( $arg =~ /MARC-?8/i ) {
+            substr($leader,9,1) = ' ';
+        }
+        $self->leader($leader);
+    }
+
+    return substr($leader,9,1) eq 'a' ? 'UTF-8' : 'MARC-8';
+}
+
 =head2 set_leader_lengths( $reclen, $baseaddr )
 
 Internal function for updating the leader's length and base address.
@@ -510,6 +561,9 @@ sub set_leader_lengths {
     my $self = shift;
     my $reclen = shift;
     my $baseaddr = shift;
+    if ($reclen > 99999) {
+            carp( "Record length of $reclen is larger than the MARC spec allows (99999 bytes)." );
+    }
     substr($self->{_leader},0,5)  = sprintf("%05d",$reclen);
     substr($self->{_leader},12,5) = sprintf("%05d",$baseaddr);
     # MARC21 defaults: http://www.loc.gov/marc/bibliographic/ecbdldrd.html
@@ -543,14 +597,17 @@ sub clone {
     my $self = shift;
     my @keeper_tags = @_;
 
-    my $clone = $self->new();
+    # create a new object of whatever type we happen to be
+    my $class = ref( $self );
+    my $clone = $class->new();
+
     $clone->{_leader} = $self->{_leader};
 
     my $filtered = @keeper_tags ? [$self->field( @keeper_tags )] : undef;
 
     for my $field ( $self->fields() ) {
         if ( !$filtered || (grep {$field eq $_} @$filtered ) ) {
-            $clone->add_fields( $field->clone );
+            $clone->append_fields( $field->clone );
         }
     }
 
@@ -638,7 +695,7 @@ sub add_fields {
             last; # Bail out, we're done eating parms
 
         # User handed us an object.
-        } elsif ( ref($parm) eq "MARC::Field" ) {
+        } elsif ( UNIVERSAL::isa($parm, 'MARC::Field') ) {
             push( @$fields, $parm );
             ++$nfields;
 
@@ -718,13 +775,14 @@ decide if MARC::Record is for you.
 
 =head1 RELATED MODULES
 
-L<MARC::Record>, L<MARC::Lint>
+L<MARC::Field>, L<MARC::Batch>, L<MARC::File::XML>, L<MARC::Charset>, 
+L<MARC::Lint>
 
 =head1 SEE ALSO
 
 =over 4
 
-=item * perl4lib (L<http://www.rice.edu/perl4lib/>)
+=item * perl4lib (L<http://perl4lib.perl.org/>)
 
 A mailing list devoted to the use of Perl in libraries.
 
@@ -738,7 +796,7 @@ The definitive source for all things MARC.
 Online version of the free booklet.  An excellent overview of the MARC format.  Essential.
 
 
-=item * Tag Of The Month (L<http://www.tagofthemonth.com/>)
+=item * Tag Of The Month (L<http://www.follettsoftware.com/sub/tag_of_the_month/>)
 
 Follett Software Company's
 (L<http://www.fsc.follett.com/>) monthly discussion of various MARC tags.
@@ -759,28 +817,19 @@ Combine MARC.pm and MARC::* into one distribution.
 
 Imagine something like this:
 
-  my @sears_headings = $marc->tag_grep( /Sears/ );
+  my @sears_headings = $marc->tag_grep( qr/Sears/ );
 
 (from Mike O'Regan)
 
 =item * Insert a field in an arbitrary place in the record
 
-=item * Allow deleting a field
-
-  for my $field ( $record->field( "856" ) ) {
-        $record->delete_field( $field ) unless useful($field);
-        } # for
-
-(from Anne Highsmith hismith@tamu.edu)
-
-
 =item * Modifying an existing field
 
 =back
 
 =head1 BUGS, WISHES AND CORRESPONDENCE
 
-Please feel free to email me at C<< <andy@petdance.com> >>.  I'm glad
+Please feel free to email me at C<< <mrylander@gmail.com> >>.  I'm glad
 to help as best I can, and I'm always interested in bugs, suggestions
 and patches.
 
@@ -803,7 +852,7 @@ Ideas are things that have been considered, but nobody's actually asked for.
 
 =item * Create multiple output formats.
 
-These could be ASCII, XML, or MarcMaker.
+These could be ASCII or MarcMaker.
 
 =back
 
@@ -814,9 +863,19 @@ This code may be distributed under the same terms as Perl itself.
 Please note that these modules are not products of or supported by the
 employers of the various contributors to the code.
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Andy Lester, C<< <andy@petdance.com> >>
+=over 4
+
+=item * Andy Lester 
+
+=item * Mike O'Regan
+
+=item * Ed Summers
+
+=item * Mike Rylander
+
+=back
 
 =cut
 
